import { computed, getCurrentInstance, reactive, ref, toRefs } from 'vue'

import { DefParams, CrudState } from './DefConfig'
import { DataMerger, DeepClone, isObjEqual } from './Methods'
import { useHooks, hooksName, HooksNameType } from './Hooks'
import { ElMessage, ElMessageBox, ElNotification } from 'element-plus'

/** 经过Init函数初始化后去除响应式的DefParams值 */
export let Params: any = {}
const CrudRequest = ref()
/** 系统培训初始化函数 */
export const InitSys = (Options: any) => {
    CrudRequest.value = Options.CrudRequest
    DefParams.value = DataMerger(DefParams.value, Options)
    CrudDebug('【初始化Options】', Options)
    // 去除响应式
    Params = { ...DefParams.value }
}

/** CRUD主函数 */
export const UseCrud = () => {
    const { props, slots, emit, refs } = getCurrentInstance() as any

    if (JSON.stringify(props) !== '{}') {
        // 初始化搜索、编辑参数
        initElementArrs(props.source)
        // 根据 props 传递过来的参数，进行重置系统参数
        DefParams.value = DataMerger(DefParams.value, props.source.options)
        DefParams.value = DataMerger(DefParams.value, props)
        CrudDebug('UseCrudStates传参【props】', props)
        CrudDebug('【DefParams】', DefParams.value)
        CrudDebug('【CrudStates】', CrudState.value)
    }
    /** 发起请求时的主键处理 */
    const KeyHandling = (data: any) => {
        let keys: any = {}
        if (props.rowKey) {
            keys[props.rowKey] = data[props.rowKey]
        }
        if (props.rowKeys) {
            props.rowKeys.forEach((key: any) => {
                keys[key] = data[key]
            })
        }
        return keys
    }
    const CRUD_Notification = (title: string, type: 'success' | 'warning' | 'info' | 'error' = 'success') => {
        ElNotification({
            title,
            type,
            offset: 70,
            duration: 2500
        })
    }

    const { hookHandler } = useHooks(props.hooks)

    /** 查询函数 */
    function queryRecord() {
        CrudState.value.tableLoading = true

        /** 查询请求携带的数据 */
        const Data = reactive({ ...CrudState.value.initSearchModel, ...CrudState.value.searchModel })
        /**
         * 处理查询时，值的问题
         * 字符串：当单行输入框有值时: 点击清空(删除)按钮 => 这个参数为空字符串，并携带到请求（查询数据无异常）
         * 选择器：当下拉选择器有值时: 点击清空(删除)按钮 => 这个参数为空字符串，并携带到请求（查询结果错误）
         */
        for (const key in Data) {
            if (Object.prototype.hasOwnProperty.call(Data, key)) {
                // 处理下拉选择器清空后，携带空字符串发起请求的问题
                if (typeof Data[key] === 'string') {
                    Data[key] = Data[key].trim()
                    if (Data[key] === '') {
                        Data[key] = undefined
                    }
                }
            }
        }
        // 查询前钩子
        if (!hookHandler(hooksName.beforeQueryHandler, { CrudState, Data })) {
            return
        }

        // 开始查询
        CrudRequest.value({
            url: props.url,
            name: DefParams.value.ApiName!.query,
            data: {
                pagenum: DefParams.value.Page_PageParams.current, //当前页
                pagesize: DefParams.value.Page_PageParams.pageSize, //分页大小
                ...Data // 查询参数
            }
        })
            .then((Res: { data?: any; page?: any }) => {
                /** 解构出后端返回的 data数组 与 page分页参数 */
                const { data, page } = Res
                // 处理响应的数据（表格赋值等）
                setTimeout(() => {
                    // 为了保证钩子的数据格式统计，加了一层
                    responseData(data, Data)
                    // 更新分页数据
                    DefParams.value.Page_PageParams.total = page.total
                    // 关闭表格加载
                    CrudState.value.tableLoading = false
                }, 50)
            })
            .catch(() => {
                CrudState.value.tableLoading = false
            })
    }
    /** 查询后 响应数据处理 */
    const responseData = (Data: any, reqDataSource: any) => {
        // 如果请求的是原始数据，就存下来（原始数据判断依据：请求没有添加任何参数）
        let originalArray = Object.keys(reqDataSource).filter((r) => reqDataSource[r] != undefined)
        if (originalArray.length == 0) {
            CrudState.value.originalData = Data
        }
        CrudState.value.originalData = DeepClone(Data)

        // 查询后钩子
        if (!hookHandler(hooksName.afterQueryHandler, { CrudState, Data })) {
            return
        }

        // 表格赋值
        CrudState.value.tableData = DeepClone(Data)
    }

    /** 重置搜索表单 */
    const refresh = () => {
        CrudState.value.searchModel = DeepClone(CrudState.value.initSearchModel)
        // queryRecord()
    }
    /** 重置编辑表单 */
    const resetForm = (formEl: any) => {
        if (!formEl) return
        formEl.resetFields()

        // 重置上传的文件
        if (IsUploadEditor.value.length) {
            IsUploadEditor.value.forEach((key) => {
                const ref = key + 'UploadRef'
                const Ref: any = refs[ref]
                Ref[0].clearFiles()
            })
        }
    }

    /**点击新增按钮，打开编辑对话框，并初始化值 */
    function addRecord() {
        // 初始化新增时表单的数据
        CrudState.value.editModel = DeepClone(CrudState.value.initEditModel)
        const Data = reactive({ ...CrudState.value.editModel })

        if (!hookHandler(hooksName.beforeAddHandler, { CrudState, Data })) {
            return
        }
        CrudState.value.Mode = 'add'
        CrudState.value.DialogVisible = true
    }

    /**点击编辑按钮，打开编辑对话框【把请求到的值赋值给 editModel】 */
    const editRecord = (row: any) => {
        // // 初始化新增时表单的数据
        // editModel.value = editFormValue.value = DeepClone(initEditModel.value)
        // 初始化新增时表单的数据

        // 获取到当前行的数据
        const Data = reactive({
            ...row
        })

        if (!hookHandler(hooksName.beforeEditHandler, { CrudState, Data })) {
            return
        }

        const DeepData = DeepClone(Data)
        // 赋值给编辑表单
        Object.keys(DeepData).forEach((key) => {
            CrudState.value.editModel[key] = DeepData[key]
            CrudState.value.editFormValue[key] = DeepData[key]
        })

        CrudState.value.Mode = 'edit'
        CrudState.value.DialogVisible = true
    }

    /**处理表单提交动作 */
    const handleSubmit = async (formEl: any) => {
        if (!formEl) return
        await formEl.validate((valid: any) => {
            if (valid) {
                const Data = reactive({
                    ...CrudState.value.editModel
                })

                if (!hookHandler(hooksName.beforeSubmitHandler, { CrudState, Data })) {
                    return
                }

                // 判断表单提交操作是新增还是编辑
                if (CrudState.value.Mode === 'add') {
                    handleAdd(Data)
                } else {
                    handleEdit(Data)
                }
            }
        })
    }
    /**处理表单新增动作 */
    const handleAdd = (value: any) => {
        CrudState.value.submitLoading = true

        let data = value

        // 判断编辑是否有 FormData 对象
        if (value.FormData instanceof FormData) {
            data = value.FormData
        }

        CrudRequest.value({ url: props.url, name: DefParams.value.ApiName!.add, data })
            .then(() => {
                CrudState.value.DialogVisible = false
                CRUD_Notification('新增成功')
                queryRecord()
                CrudState.value.submitLoading = false
            })
            .catch(() => {
                // 请求数据时，后端返回错误代码
                CrudState.value.submitLoading = false
            })
    }
    /**处理表单编辑动作 */
    const handleEdit = (value: any) => {
        CrudState.value.submitLoading = false

        /** 最终提交时的参数 */
        let data = value

        // 判断编辑是否有 FormData 对象
        if (value.FormData instanceof FormData) {
            data = value.FormData
        } else {
            // 主要解决了编辑表单时，只提交修改项
            const Equal = isObjEqual(CrudState.value.editModel, data)
            // 没有编辑项时，就不发送请求
            if (JSON.stringify(Equal) === '{}') {
                CrudState.value.submitLoading = false
                return ElMessage({ message: '没有提交项！', grouping: true })
            }

            data = {
                ...KeyHandling(value),
                ...Equal
            }
        }

        CrudRequest.value({ url: props.url, name: DefParams.value.ApiName!.edit, data })
            .then(() => {
                CrudState.value.DialogVisible = false
                CRUD_Notification('编辑成功')
                queryRecord()
                CrudState.value.submitLoading = false
            })
            .catch(() => {
                // 请求数据时，后端返回错误代码
                CrudState.value.submitLoading = false
            })
    }
    /**标记删除 */
    const delFlag = (row: any) => {
        ElMessageBox.confirm(`确定要 “${row.deleteFlag ? '启用' : '禁用'}” 该条数据吗？`, '提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
        })
            .then(() => {
                const Data = reactive({
                    row: { ...row },
                    ...KeyHandling(row),
                    deleteFlag: row.deleteFlag == 0 ? 1 : 0
                })
                if (!hookHandler(hooksName.beforedisabledHandler, { CrudState, Data })) {
                    return
                }
                CrudRequest.value({
                    url: props.url,
                    name: DefParams.value.ApiName!.deleteFlag,
                    data: Data
                }).then(() => {
                    CRUD_Notification(`数据${row.deleteFlag ? '启用' : '禁用'}成功`)
                    queryRecord()
                })
            })
            .catch(() => {})
    }
    /**删除单行数据 */
    const deleteRecord = (row: any) => {
        ElMessageBox.confirm('确定要删除本条数据吗？', '提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
        })
            .then(() => {
                // 获取到需要删除的当前行的数据
                const Data = reactive({
                    row: { ...row },
                    ...KeyHandling(row)
                })

                // 删除前钩子
                if (!hookHandler(hooksName.beforeDeteleHandler, { CrudState, Data })) {
                    return
                }
                CrudRequest.value({
                    url: props.url,
                    name: DefParams.value.ApiName!.delete,
                    data: Data
                }).then(() => {
                    CRUD_Notification('删除成功')
                    queryRecord()
                })
            })
            .catch(() => {})
    }
    /**删除多行数据 */
    const deleteMulRecord = () => {
        ElMessageBox.confirm(`确定要删除这几条数据吗？`, '提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning'
        })
            .then(() => {
                // 整理批量删除的 id
                const ids = CrudState.value.multipleSelection.map((selection: any) => {
                    return selection[props.rowKey!]
                })
                CrudRequest.value({
                    url: props.url,
                    name: DefParams.value.ApiName!.remove,
                    data: { [props.rowKey!]: ids, selection: CrudState.value.multipleSelection }
                }).then(() => {
                    CRUD_Notification('批量删除成功')
                    CrudState.value.multipleSelection = []
                    //把已经选择的数据清空
                    CrudState.value.CompRef.TableRef.clearSelection()
                    //重新查找
                    queryRecord()
                })
            })
            .catch(() => {})
    }

    return {
        queryRecord,
        refresh,
        resetForm,

        addRecord,
        editRecord,
        handleSubmit,
        delFlag,
        deleteRecord,
        deleteMulRecord
    }
}

export function UseParam() {
    return { ...toRefs(DefParams.value) }
}
export function UseState() {
    return { ...toRefs(CrudState.value) }
}

/** DeBug */
export function CrudDebug(...params: any) {
    if (DefParams.value.debug) {
        console.info('%c【DeBug】', 'color:red', ...params)
    }
}

/** 初始化搜索、编辑参数 */
function initElementArrs(source: SourceOptType) {
    const { filter, editor, column, columns } = source

    // 初始化搜索框数据（如果options设置了默认值，会自动携带上，会一直在。）
    let filterModel: any = {}
    filter.forEach((item) => {
        filterModel[item] = columns[item].searchDefaultValue
        CrudState.value.ElementArrs.filter[item] = columns[item]
    })
    CrudState.value.initSearchModel = DeepClone(filterModel)
    //把初始化的值进行赋值
    CrudState.value.searchModel = DeepClone(CrudState.value.initSearchModel)

    // 初始化编辑框数据（新增时会自动输入的默认值）
    let editorModel: any = {}
    editor.forEach((item) => {
        editorModel[item] = columns[item].addDefaultValue
        CrudState.value.ElementArrs.editor[item] = columns[item]
    })
    CrudState.value.initEditModel = DeepClone(editorModel)

    // 表格数据行
    column.forEach((item) => {
        CrudState.value.ElementArrs.column[item] = columns[item]
    })

    // TODO:展示的表格列
    // CrudStates.value.tableColumn = columns
}

/** 判断编辑中 是否有上传功能 */
const IsUploadEditor = computed(() => {
    let Keys: string[] = []
    // 如果编辑有图片预览，就自动绑定
    for (const key in CrudState.value.ElementArrs.editor) {
        if (Object.prototype.hasOwnProperty.call(CrudState.value.ElementArrs.editor, key)) {
            const item = CrudState.value.ElementArrs.editor[key]
            if (item.type === 'el-upload') {
                // 只要是上传，无论是生命卡片样式，都需要清空
                Keys.push(key)
            }
        }
    }
    return Keys
})
